@@ -1,21 +1,20 @@
 from __future__ import print_function, division, absolute_import
-from gevent import monkey
 
-monkey.patch_all()
+from ..config import config
+from .wc import WebsocketClient
+from .data_adapter import DataAdapter
+from .methods import methods
+from .messages import messages
+from hkube_python_wrapper.tracing import Tracer
+from hkube_python_wrapper.codeApi.hkube_api import HKubeApi
+from hkube_python_wrapper.communication.DataServer import DataServer
+from hkube_python_wrapper.util.queueImpl import Queue, Empty
 import os
 import sys
 import importlib
 import traceback
-import gevent
 from events import Events
-from hkube_python_wrapper.communication.DataServer import DataServer
-from hkube_python_wrapper.codeApi.hkube_api import HKubeApi
-from hkube_python_wrapper.tracing import Tracer
-from .messages import messages
-from .methods import methods
-from .data_adapter import DataAdapter
-from .wc import WebsocketClient
-from ..config import config
+from threading import Thread, Timer
 
 
 class Algorunner:
@@ -27,12 +26,14 @@ class Algorunner:
         self._loadAlgorithmError = None
         self._connected = False
         self._hkubeApi = None
+        self._msg_queue = Queue()
         self._dataAdapter = None
         self._dataServer = None
         self._discovery = None
         self._wsc = None
         self.tracer = None
         self._storage = None
+        self._active = True
 
     @staticmethod
     def Run(start=None, init=None, stop=None, exit=None, options=None):
@@ -57,7 +58,8 @@ class Algorunner:
         else:
             algorunner.loadAlgorithm(config)
         jobs = algorunner.connectToWorker(config)
-        gevent.joinall(jobs)
+        for j in jobs:
+            j.join()
 
     @staticmethod
     def Debug(debug_url, start, init=None, stop=None, exit=None, options=None):
@@ -67,7 +69,8 @@ class Algorunner:
         config.discovery["enable"] = False
         algorunner.loadAlgorithmCallbacks(start, init=init, stop=stop, exit=exit, options=options or config)
         jobs = algorunner.connectToWorker(config)
-        gevent.joinall(jobs)
+        for j in jobs:
+            j.join()
 
     def loadAlgorithmCallbacks(self, start, init=None, stop=None, exit=None, options=None):
         try:
@@ -153,15 +156,43 @@ class Algorunner:
         self._url += '?storage={storage}&encoding={encoding}'.format(
             storage=self._storage, encoding=encoding)
 
-        self._wsc = WebsocketClient(encoding)
+        self._wsc = WebsocketClient(self._msg_queue, encoding, self._url)
         self._initStorage(options)
-        self._hkubeApi = HKubeApi(self._wsc, self._dataAdapter, self._storage)
+        self._hkubeApi = HKubeApi(self._wsc, self, self._dataAdapter, self._storage)
         self._registerToWorkerEvents()
 
         print('connecting to {url}'.format(url=self._url))
-        job1 = gevent.spawn(self._wsc.startWS, self._url)
-        job2 = gevent.spawn(self._dataServer and self._dataServer.listen)
-        return [job1, job2]
+        self._wsc.start()
+        self._dataServer and self._dataServer.listen()
+        runThread = Thread(name="RunThread", target=self.run)
+        runThread.start()
+        return [self._wsc, runThread]
+
+    def handle(self, command, data):
+        if (command == messages.incoming.initialize):
+            self._init(data)
+        if (command == messages.incoming.start):
+            self._start(data)
+        if (command == messages.incoming.stop):
+            self._stop(data)
+        if (command == messages.incoming.exit):
+            self._exit(data)
+        if (command in [messages.incoming.algorithmExecutionDone, messages.incoming.algorithmExecutionError]):
+            self._hkubeApi.algorithmExecutionDone(data)
+        if (command in [messages.incoming.subPipelineDone, messages.incoming.subPipelineError, messages.incoming.subPipelineStopped]):
+            self._hkubeApi.subPipelineDone(data)
+
+    def get_message(self, blocking=True):
+        return self._msg_queue.get(block=blocking, timeout=0.1)
+
+    def run(self):
+        while self._active:
+            try:
+                (command, data) = self.get_message()
+                self.handle(command, data)
+            except Empty:
+                pass
+        print('Exiting run loop')
 
     def _initStorage(self, options):
         if (self._storage != 'v1'):
@@ -182,7 +213,9 @@ class Algorunner:
         self._dataAdapter = DataAdapter(options, self._dataServer)
 
     def close(self):
-        self._wsc.shutDown()
+        if (self._wsc):
+            self._wsc.shutDown()
+        self._active = False
 
     def _registerToWorkerEvents(self):
         self._wsc.events.on_connection += self._connection
@@ -257,7 +290,6 @@ class Algorunner:
 
     def _handle_responseV2(self, algorithmData, jobId, taskId, nodeName, savePaths, span):
         encodedData = self._dataAdapter.encode(algorithmData)
-
         data = {
             'jobId': jobId,
             'taskId': taskId,
@@ -271,8 +303,8 @@ class Algorunner:
         storingData.update(storageInfo)
         incache = None
         if (self._dataServer and savePaths):
-            incache = self._dataServer.setSendingState(taskId, algorithmData, len(encodedData))
-        if(incache):
+            incache = self._dataServer.setSendingState(taskId, encodedData, len(encodedData))
+        if (incache):
             storingData.update({'discovery': self._discovery, 'taskId': taskId})
             self._sendCommand(messages.outgoing.storing, storingData)
             self._dataAdapter.setData({'jobId': jobId, 'taskId': taskId, 'data': encodedData})
@@ -284,19 +316,20 @@ class Algorunner:
         self._sendCommand(messages.outgoing.done, None)
 
     def _reportServing(self, interval=None):
-        if(interval is None):
+        if (interval is None):
             return
         interval = interval / 1000
 
         def reportInterval():
-            while (True):
-                self._reportServingStatus()
-                gevent.sleep(interval)
-        gevent.spawn(reportInterval)
+            if (not self._active):
+                return
+            self._reportServingStatus()
+            Timer(interval, reportInterval).start()
+        reportInterval()
 
     def _reportServingStatus(self):
         isServing = self._dataServer.isServing()
-        if(isServing):
+        if (isServing):
             self._sendCommand(messages.outgoing.servingStatus, True)
 
     def _stop(self, options):
